<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Services; 

use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\CMP\Util\CmpUtilities;

class Cmp07600AlteracaoAprovacaoAfService {

    protected $connection;
    protected $smtpUtil;
    protected $entityManager;
    protected $cmpUtilities;
    protected $newCode;
    protected $environment;
    
    public function __construct ($entityManager, $smtpUtil, CmpUtilities $cmpUtilities, $newCode, $environment) {
        $this->smtpUtil = $smtpUtil; 
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->cmpUtilities = $cmpUtilities;
        $this->newCode = $newCode;
        $this->environment = $environment;
    }
    
    public function procedureGeraNovoCodigo($tabela) {
        try {
            return $this->newCode->getCodeByTableName($tabela, $this->environment->getNrOrg());
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function parseFloat($floatVar) {
        if(is_string($floatVar)) {
            $floatVar = str_replace(".", "", $floatVar);    
            $floatVar = str_replace(",", ".", $floatVar);
        }
        return $floatVar;
    }
    
    public function getSystemParamGeral(){
        
        $select = "SELECT IDSISDOS FROM PARAMGERAL";

        try
        {
            $std = $this->connection->prepare($select);
            $std->execute();
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getBranchOperator($selectParams){
        
        $select = "SELECT DISTINCT CDOPERADOR
                      FROM GRUPFILIOPER, FILGRUPFILI
                     WHERE GRUPFILIOPER.CDGRUPFILI = FILGRUPFILI.CDGRUPFILI 
                       AND FILGRUPFILI.CDFILIAL    = :P_CDFILDEST";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getTradingCenter($selectParams){
        
        $select = "SELECT NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CDFILCENTCMP
                      FROM FILIAL FIL, FILIAL
                     WHERE FILIAL.CDFILIAL = :P_CDFILDEST
                       AND NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = FIL.CDFILIAL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getPermissions($selectParams){
        
        $select = "SELECT CDSUBGRUPPROD, CDGRUPPROD, IDBLOQFORNCOTAF, IDCARPRECLAF
                      FROM PARASUPRI
                     WHERE CDFILIAL = :P_CDFILCENTCMP
                       AND CDSUBGRUPPROD IS NOT NULL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getIntegration($row, $selectParams){
        
        $select = self::getIntegrationSelectByCurrentSystem($row);
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getIntegrationSelectByCurrentSystem($row){
        
        if($row["IDSISDOS"] == 3){
            
            $select = "SELECT AUTOFORNE.CDFILDEST
                        FROM AUTOFORNE, FORNECEDOR
                       WHERE AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                        AND AUTOFORNE.CDFILDEST  = :P_CDFILDEST
                        AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN
                        AND NVL(FORNECEDOR.USR_INTEGRAMESAP, 'N') = 'S'
                        AND USR_IDINTEGRAMESAP IS NOT NULL";   
            
        } else {
            
            $select = "SELECT AUTOFORNE.CDFILDEST
                          FROM AUTOFORNE
                         WHERE AUTOFORNE.CDFILDEST  = :P_CDFILDEST
                           AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN";
            
        }
        
        return $select;
        
    }
    
    public function getFrequency($selectParams){
        
        $select = "SELECT PERIENTR.CDPERIENTR, PERIENTR.NMPERIENTR
                      FROM REFORFIL, PERIENTR
                     WHERE REFORFIL.CDPERIENTR = PERIENTR.CDPERIENTR
                       AND REFORFIL.CDFORNECED = :P_CDFORNECED
                       AND REFORFIL.CDFILIAL   = :P_CDFILDEST";
                       
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function executeProcedureCarregaPeriforn($row) {
       
        $procedure = new StoredProcedure($this->connection, "CARREGA_PERIFORN");
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILDEST"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $row["CDFORNECED"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTPROGENTAF', Param::PARAM_INPUT, $row["DTNOVAENTRAF"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NEW_DTPROGENTAF', Param::PARAM_OUTPUT, null, null, 20));
        
        $newDate = $procedure->execute();
        return $newDate["P_NEW_DTPROGENTAF"];
        
    }
    
    public function sendEmailToProductResponsibleOperators($row) {
        $attrImg = 'width="200px"';
	    $srcLogo = $this->cmpUtilities->returnLogo('PORTAL E-COMMERCE');
        $this->cmpUtilities->getLogoAttributes($srcLogo, $attrImg);
        $lgpd = $this->cmpUtilities->returnLgpd($row['FILIAL_LOGADA']);
        
        $subject = 'Aviso de Alteração de AF';
        $assunto = "<p class='bold' style='text-align: center; color: #ED2424'>Aviso de Alteração de AF</p>";
        
        foreach ($row["OPERADORES_RESPONSAVEIS"] as $operator) {
            try {
                $mensagem = "";
                $mensagem .= "<p>" . $operator["NMOPERADOR"] . ", </p>";
                $mensagem .= "<p> Abaixo seguem as informações da autorização do fornecimento cuja alteração deve ser analisada por você. São elas: </p>";
                $mensagem .= "<ul>
                                <li><strong>AF: </strong>" . $row["NRAUTOFORN"] . "</li>
                                <li><strong>Unidade Lançamento: </strong>" . $row["CDFILLANC"] . "</li>
                                <li><strong>Unidade Destino: </strong>" . $row["NMFILDEST"] . "</li>
                                <li><strong>Data de Entrega: </strong>" . $row["DTPROGENTAF"] . "</li>
                                <li><strong>Comprador: </strong>" . $row["NMOPERADOR"] . "</li>
                                <li><strong>Fornecedor: </strong>" . $row["NMRAZSOCFORN"] . "</li>
                              </ul>";
                              
                $messageSendMail = $this->cmpUtilities->returnMessageSendMail($srcLogo, $assunto, $mensagem, $attrImg, $lgpd);
                $this->cmpUtilities->enviaEmail($row['EMAIL_LOGADO'], $subject, $operator["DSEMAILOPER"], $messageSendMail, $operator["DSEMAILOPER"]);
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    public function getAfResponsibleOperators($selectParams){
        
        $select = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.DSEMAILOPER, OPERADOR.CDSENEMAILOP, OPERADOR.NMOPERADOR
                      FROM GRUPFILIOPER, FILGRUPFILI, OPERADOR
                     WHERE GRUPFILIOPER.CDGRUPFILI = FILGRUPFILI.CDGRUPFILI 
                       AND FILGRUPFILI.CDFILIAL    = :P_CDFILDEST
                       AND GRUPFILIOPER.CDOPERADOR = OPERADOR.CDOPERADOR";
       
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function updateAllAfProductDeliveryDate($updateParams){
        
        $update = "UPDATE ITEMAUTF
                    SET DTNOVAENTRAF = :P_DTNOVAENTRAF
                   WHERE CDFILDEST = :P_CDFILDEST
                   AND CDFILLANC = :P_CDFILLANC
                   AND NRAUTOFORN = :P_NRAUTOFORN";
        
        try
        {
            $std = $this->connection->prepare($update);
            $std->execute($updateParams);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getProductsWithNewDeliveryDate($selectParams){
        
        $select = "SELECT QTNOVAPROGAF
                    FROM ITEMAUTF    
                   WHERE CDFILDEST = :P_CDFILDEST
                   AND CDFILLANC = :P_CDFILLANC
                   AND NRAUTOFORN = :P_NRAUTOFORN
                   AND QTNOVAPROGAF IS NOT NULL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getParcmpsup($selectParams){
        
        $select = "SELECT IDOBLIBAFMAN
                      FROM PARCMPSUP
                     WHERE CDFILIAL = :CDFILIAL
                       AND NRORG = :NRORG";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getParamSupplier($params){
        
        $query = "SELECT
                     AUTOFORNE.CDFILDEST,
                     NVL(FORNECEDOR.USR_INTEGRAMESAP,'N') USR_INTEGRAMESAP,
                     FORNECEDOR.IDPERALTAFCONF,
                     NVL(AUTOFORNE.IDSITUATENDIMENTOAF,'X') IDSITUATENDIMENTOAF
                 FROM
                     AUTOFORNE,
                     FORNECEDOR
                 WHERE
                     AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                     AND AUTOFORNE.CDFILDEST =:P_CDFILDEST
                     AND AUTOFORNE.NRAUTOFORN =:P_NRAUTOFORN";

        $std = $this->connection->prepare($query);
        $std->execute($params);
        return $std->fetchAll()[0];
        
    }
    
    public function updateAf($updateParams){
        
          $update = "UPDATE AUTOFORNE
                    SET IDAFFORALIM = :IDAFFORALIM,
                        IDSITUATUAAF = :IDSITUATUAAF "
                    . $updateParams['CDGERENTE'] .    
                   "WHERE CDFILDEST = :CDFILDEST
                   AND CDFILLANC = :CDFILLANC
                   AND NRAUTOFORN = :NRAUTOFORN
                   AND NRORG = :NRORG";
        
        $params = array(
            
            "IDAFFORALIM"   => $updateParams["IDAFFORALIM"],
            "IDSITUATUAAF"  => $updateParams["IDSITUATUAAF"],
            "CDFILDEST"     => $updateParams["CDFILDEST"],
            "CDFILLANC"     => $updateParams["CDFILLANC"],
            "NRAUTOFORN"    => $updateParams["NRAUTOFORN"],
            "NRORG"         => $updateParams["NRORG"]
            
        );
        
        try
        {
            $std = $this->connection->prepare($update);
            $std->execute($params);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function updateCdtpctpagar($updateParams) {
        $query = "UPDATE AUTOFORNE
            SET CDTPCTPAGAR =
              (SELECT PRODUTO.CDTPCTPAGAR
              FROM PRODUTO,
                (SELECT MAX(PRODUTO.CDARVPROD) AS CDARVPROD
                FROM PRODUTO,
                  (SELECT MIN(A.CDARVPROD) AS CDARVPROD
                  FROM
                    (SELECT CDARVPROD
                    FROM ITEMAUTF,
                      PRODUTO P
                    WHERE P.CDPRODUTO       = ITEMAUTF.CDPRODUTO
                    AND ITEMAUTF.CDFILDEST  = :CDFILDEST
                    AND ITEMAUTF.CDFILLANC  = :CDFILLANC
                    AND ITEMAUTF.NRAUTOFORN = :NRAUTOFORN
                    AND ITEMAUTF.NRORG = :NRORG
                    GROUP BY CDARVPROD
                    HAVING MAX(QTPROGENTAF   *VRPREUNIAF) =
                      (SELECT MAX(QTPROGENTAF*VRPREUNIAF)
                      FROM ITEMAUTF
                      WHERE ITEMAUTF.CDFILDEST = :CDFILDEST
                      AND ITEMAUTF.CDFILLANC   = :CDFILLANC
                      AND ITEMAUTF.NRAUTOFORN  = :NRAUTOFORN
                      AND ITEMAUTF.NRORG = :NRORG
                      )
                    )A
                  ) ITEMAUTF
                WHERE ITEMAUTF.CDARVPROD LIKE PRODUTO.CDARVPROD
                  || '%'
                AND PRODUTO.CDTPCTPAGAR IS NOT NULL
                AND PRODUTO.NRORG = :NRORG
                ) TIPOCTPAGAR
              WHERE PRODUTO.CDARVPROD  = TIPOCTPAGAR.CDARVPROD
              AND PRODUTO.CDTPCTPAGAR IS NOT NULL
              AND PRODUTO.NRORG = :NRORG
              )
            WHERE AUTOFORNE.CDFILDEST  = :CDFILDEST
            AND AUTOFORNE.CDFILLANC    = :CDFILLANC
            AND AUTOFORNE.NRAUTOFORN   = :NRAUTOFORN
            AND AUTOFORNE.NRORG = :NRORG
            AND AUTOFORNE.CDTPCTPAGAR IS NULL";
            
        $params = array(
            "CDFILDEST"  => $updateParams["CDFILDEST"],
            "CDFILLANC"  => $updateParams["CDFILLANC"],
            "NRAUTOFORN" => $updateParams["NRAUTOFORN"],
            "NRORG"      => $updateParams["NRORG"]
        );
        $result = $this->entityManager->getConnection()->executeUpdate($query, $params);
        return $result;
    }
        
    public function CUSTO_AF($procParams) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'CUSTO_AF');
        $procedure->addParam(new Param('P_CDFILDEST',   Param::PARAM_INPUT, $procParams["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',   Param::PARAM_INPUT, $procParams["CDFILLANC"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT, $procParams["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPURAF', Param::PARAM_INPUT, $procParams["NRSEQAPURAF"], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function IMPOSTO_ITEMAF($row) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'IMPOSTO_ITEMAF');
        $procedure->addParam(new Param('P_EMPRESA',    Param::PARAM_INPUT, $row["CDEMPRESA"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',   Param::PARAM_INPUT, $row["CDINSCESTA"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FORNECEDOR', Param::PARAM_INPUT, $row["CDFORNECED"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILDEST',    Param::PARAM_INPUT, $row["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILLANC',    Param::PARAM_INPUT, $row["CDFILLANC"],   Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_LANCTOAF',   Param::PARAM_INPUT, $row["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',   Param::PARAM_INPUT, $row["DTPROGENTAF"], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function adqEntrItmSol($row) {
        $param = array();
        $param["P_CDFILDEST"]       = $row["CDFILDEST"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_NRAUTOFORN"]      = $row["NRAUTOFORN"];
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_DTPROGENTAF"]     = substr($row["DTPROGENTAF"], 0, 10);
        
        $query = "SELECT
                      MIN(DTENTRITMSOL) DTENTRITMSOL
                    FROM
                      ITEMAUTF
                    WHERE
                      (CDFILDEST  = :P_CDFILDEST)   AND
                      (CDFILLANC  = :P_CDFILLANC)   AND
                      (NRAUTOFORN = :P_NRAUTOFORN)  AND
                      (CDPRODUTO  = :P_CDPRODUTO)   AND
                      (CDMARCA    = :P_CDMARCA)     AND
                      (DTPROGENTAF= :P_DTPROGENTAF)";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqItemSoli($row, $adqEntrItmSol) {
        $param = array();
        $param["P_CDFILSOLI"]   = $row["CDFILDEST"];
        $param["P_DTPROGENTR"]  = count($adqEntrItmSol) > 0 ? substr($adqEntrItmSol[0]['DTENTRITMSOL'], 0, 10) : substr($row['DTPROGENTAF'], 0 , 10);
        $param["P_CDPRODUTO"]   = $row["NRAUTOFORN"];
        
        $query = "SELECT
                      ITEMSOLI.CDFILSOLI,
                      ITEMSOLI.CDFILLAN,
                      ITEMSOLI.NRSOLICMP,
                      ITEMSOLI.CDPRODUTO,
                      ITEMSOLI.DTPROGENTR,
                      ITEMSOLI.QTSOLIPROD,
                      PRODSOLI.CDPRODESTO,
                      NVL(PRODSOLI.VRFATOCONV, 1) AS VRFATOSOLI,
                      NVL(PRODSOLI.VRARREDCMP, 1) VRARREDSOLI,
                      NVL(PRODUTO.VRFATOCONV, 1) VRFATOCONV,
                      NVL(PRODUTO.VRARREDCMP, 1) VRARREDCMP
                    FROM
                      ITEMSOLI,
                      PRODUTO PRODSOLI,
                      PRODUTO
                    WHERE
                      ITEMSOLI.CDPRODUTO   = PRODSOLI.CDPRODUTO AND
                      ITEMSOLI.CDFILSOLI   = :P_CDFILSOLI       AND
                      ITEMSOLI.DTPROGENTR  = :P_DTPROGENTR      AND
                      PRODUTO.CDPRODUTO    = :P_CDPRODUTO       AND
                      PRODSOLI.CDPRODESTO  = PRODUTO.CDPRODESTO AND
                      NVL(ITEMSOLI.IDSOLIAPROV, 'S') = 'S'
                    ORDER BY
                       ITEMSOLI.NRSOLICMP";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpQtSoliProd($row, $qntd) {
        $query = "update ITEMSOLI
                    set
                      QTSOLIPROD = QTSOLIPROD + :P_QTSOLIPROD
                    where
                      CDFILSOLI        = :P_CDFILSOLI         AND
                      CDFILLAN         = :P_CDFILLAN          AND
                      NRSOLICMP     = :P_NRSOLICMP      AND
                      CDPRODUTO   = :P_CDPRODUTO    AND
                      DTPROGENTR = :P_DTPROGENTR";
            
        $params = array(
            "P_QTSOLIPROD" => $qntd,
            "P_CDFILSOLI"  => $row["CDFILSOLI"],
            "P_CDFILLAN"   => $row["CDFILLAN"],
            "P_NRSOLICMP"  => $row["NRSOLICMP"],
            "P_CDPRODUTO"  => $row["CDPRODUTO"],
            "P_DTPROGENTR" => substr($row["DTPROGENTR"], 0, 10)
        );
        
        $result = $this->entityManager->getConnection()->executeUpdate($query, $params);
        return $result;
    }
    
    public function acmInsMovOpSol($row, $msg, $qtdOper) {
        $param = array(
            'P_CDFILSOLMOV' => $row['CDFILSOLI'],
            'P_CDFILLANMOV' => $row['CDFILLAN'],
            'P_NRSOLIMOV'   => $row['NRSOLICMP'],
            'P_CDPRODUTO'   => $row['CDPRODUTO'],
            'P_DTENTROPER'  => substr($row['DTPROGENTR'], 0, 10),
            'P_CDOPERADOR'  => $this->environment->getCdOperador(),
            'P_QTANTOPER'   => $row['QTSOLIPROD'],
            'P_QTDEPOPER'   => $qtdOper,
            'P_DSJUSTOPER'  => $msg
        );
        
        $query = "INSERT INTO MOVOPSOL
                    (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV,
                     CDPRODUTO, DTENTROPER, DTOPERSOL,
                     CDOPERADOR, QTANTOPER, QTDEPOPER,
                    DSJUSTOPER)
                    VALUES
                    (:P_CDFILSOLMOV, :P_CDFILLANMOV, :P_NRSOLIMOV,
                     :P_CDPRODUTO, :P_DTENTROPER, SYSDATE,
                     :P_CDOPERADOR, :P_QTANTOPER, :P_QTDEPOPER,
                    :P_DSJUSTOPER)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmDelItemSoli($row) {
        $param = array(
            'P_CDFILSOLI'   => $row['CDFILSOLI'],
            'P_CDFILLAN'    => $row['CDFILLAN'],
            'P_NRSOLICMP'   => $row['NRSOLICMP'],
            'P_CDPRODUTO'   => $row['CDPRODUTO'],
            'P_DTPROGENTR'  => substr($row['DTPROGENTR'], 0, 10)
        );
        
        $query = "DELETE FROM ITEMSOLI
                    where
                      CDFILSOLI        = :P_CDFILSOLI         AND
                      CDFILLAN         = :P_CDFILLAN          AND
                      NRSOLICMP     = :P_NRSOLICMP      AND
                      CDPRODUTO   = :P_CDPRODUTO    AND
                      DTPROGENTR = :P_DTPROGENTR";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function adqAux_Imposto($row) {
        $param = array();
        $param["P_CDFILDEST"]   = $row["CDFILDEST"];
        $param["P_CDFILLANC"]   = $row["CDFILLANC"];
        $param["P_NRAUTOFORN"]  = $row["NRAUTOFORN"];
        $param["P_CDPRODUTO"]   = $row["CDPRODUTO"];
        $param["P_CDMARCA"]     = $row["CDMARCA"];
        $param["P_DTPROGENTAF"] = substr($row["DTPROGENTAF"], 0, 10);
        
        $query = "SELECT
                      COUNT(*) AS CTIMP,
                      CDFILDEST,    CDFILLANC,
                      NRAUTOFORN,   CDPRODUTO,
                      CDMARCA,      DTPROGENTAF,
                      CDIMPOSTO,    VRIMPITMAF,
                      VRPERREDBCAF, VRBASECALCAF,
                      VRALIQITMAF,  VRPARCDEDAF,
                      CDFISCITMAF,  VRADIMPAF,
                      IDIMPDIGAF
                    FROM
                      IMPITMAF
                    WHERE
                      IMPITMAF.CDFILDEST   = :P_CDFILDEST   AND
                      IMPITMAF.CDFILLANC   = :P_CDFILLANC   AND
                      IMPITMAF.NRAUTOFORN  = :P_NRAUTOFORN  AND
                      IMPITMAF.CDPRODUTO   = :P_CDPRODUTO   AND
                      IMPITMAF.CDMARCA     = :P_CDMARCA     AND
                      IMPITMAF.DTPROGENTAF = :P_DTPROGENTAF
                    GROUP BY
                      CDFILDEST,    CDFILLANC,
                      NRAUTOFORN,   CDPRODUTO,
                      CDMARCA,      DTPROGENTAF,
                      CDIMPOSTO,    VRIMPITMAF,
                      VRPERREDBCAF, VRBASECALCAF,
                      VRALIQITMAF,  VRPARCDEDAF,
                      CDFISCITMAF,  VRADIMPAF,
                      IDIMPDIGAF";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmDelImpItm($row) {
        $param = array(
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_CDMARCA'       => $row['CDMARCA']
        );
        
        $query = "DELETE FROM IMPITMAF
                    WHERE CDFILDEST = :OLD_CDFILDEST AND
                                   CDFILLANC = :OLD_CDFILLANC AND
                                   NRAUTOFORN = :OLD_NRAUTOFORN AND
                                   DTPROGENTAF = :OLD_DTPROGENTAF AND
                                   CDPRODUTO = :OLD_CDPRODUTO AND
                                   CDMARCA = :OLD_CDMARCA";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function adqExisteItemautf($row) {
        $param = array();
        $param["OLD_CDFILDEST"]  = $row["CDFILDEST"];
        $param["OLD_CDFILLANC"]  = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"] = $row["NRAUTOFORN"];
        $param["OLD_CDPRODUTO"]  = $row["CDPRODUTO"];
        $param["OLD_CDMARCA"]    = $row["CDMARCA"];
        $param["DTPROGENTAF"]    = substr($row["DTPROGENTAF"], 0, 10);
        
        $query = "SELECT ITEMAUTF.QTPROGENTAF
                  FROM AUTOFORNE, ITEMAUTF, FORNECEDOR
                 WHERE AUTOFORNE.CDFILDEST   = ITEMAUTF.CDFILDEST 
                   AND AUTOFORNE.CDFILLANC   = ITEMAUTF.CDFILLANC 
                   AND AUTOFORNE.NRAUTOFORN  = ITEMAUTF.NRAUTOFORN
                   AND AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                   AND ITEMAUTF.CDFILDEST     = :OLD_CDFILDEST
                   AND ITEMAUTF.CDFILLANC     = :OLD_CDFILLANC        
                   AND ITEMAUTF.NRAUTOFORN    = :OLD_NRAUTOFORN  
                   AND ITEMAUTF.CDMARCA       = :OLD_CDMARCA         
                   AND ITEMAUTF.CDPRODUTO     = :OLD_CDPRODUTO   
                   AND ITEMAUTF.DTPROGENTAF   = :DTPROGENTAF";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmDelItemSoliRelacAF($row) {
        $param = array(
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_CDMARCA'       => $row['CDMARCA']
        );
        
        $query = "delete from ITEMSOLIRELACAF
                    where
                      CDFILDEST = :OLD_CDFILDEST and
                      CDFILLANC = :OLD_CDFILLANC and
                      NRAUTOFORN = :OLD_NRAUTOFORN and
                      CDPRODUTOAF = :OLD_CDPRODUTO and
                      DTPROGENTAF = :OLD_DTPROGENTAF and
                      CDMARCAAF = :OLD_CDMARCA";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmUpdItm($row, $tipo, $msgJust, $itemAf) {
        if($itemAf != null) {
            $qtd = self::parseFloat($itemAf[0]['QTPROGENTAF']) + $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG']);
        } else {
            $qtd = $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG']);
        }
        
        $param = array(
            'QTPROGENTAF'       => $tipo == 'A' ? $qtd : self::parseFloat($row['QTORIG']),
            'DTPROGENTAF'       => $tipo == 'A' ? substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'QTNOVAPROGAF'      => null,
            'DSJUSTITMAF'       => null,
            'DTNOVAENTRAF'      => null,
            'CDOPERAPROV'       => $this->environment->getCdOperador(),
            'P_JUSTREPROV'      => $msgJust,
            'P_IDTIPOLANITM'    => $tipo == 'A' ? 'M' : null,
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_CDMARCA'       => $row['CDMARCA'],
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10)
        );
        
        $query = "update ITEMAUTF
                    set
                      QTPROGENTAF    = :QTPROGENTAF, 
                      DTPROGENTAF    = :DTPROGENTAF,
                      QTNOVAPROGAF = :QTNOVAPROGAF,
                      DSJUSTITMAF      = :DSJUSTITMAF,
                      DTNOVAENTRAF = :DTNOVAENTRAF,
                      CDOPERAPROV = :CDOPERAPROV,
                      DTHRAPROVAF = SYSDATE,
                      DSJUSTITMAFRPV  = :P_JUSTREPROV,
                      IDTIPOLANITM    = :P_IDTIPOLANITM
                    where
                      CDFILDEST        = :OLD_CDFILDEST        AND
                      CDFILLANC        = :OLD_CDFILLANC        AND
                      NRAUTOFORN  = :OLD_NRAUTOFORN  AND
                      CDMARCA          = :OLD_CDMARCA         AND
                      CDPRODUTO     = :OLD_CDPRODUTO   AND
                      DTPROGENTAF = :OLD_DTPROGENTAF";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmDelItemAutF($row) {
        $param = array(
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_CDMARCA'       => $row['CDMARCA']
        );
        
        $query = "delete from ITEMAUTF
                    where
                      CDFILDEST = :OLD_CDFILDEST and
                      CDFILLANC = :OLD_CDFILLANC and
                      NRAUTOFORN = :OLD_NRAUTOFORN and
                      CDPRODUTO = :OLD_CDPRODUTO and
                      DTPROGENTAF = :OLD_DTPROGENTAF and
                      CDMARCA = :OLD_CDMARCA";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsImpItm($row, $imp) {
        $param = array(
            'CDFILDEST'     => $row['CDFILDEST'],
            'CDFILLANC'     => $row['CDFILLANC'],
            'NRAUTOFORN'    => $row['NRAUTOFORN'],
            'CDPRODUTO'     => $row['CDPRODUTO'],
            'CDMARCA'       => $row['CDMARCA'],
            'DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'CDIMPOSTO'     => $imp['CDIMPOSTO'],
            'VRIMPITMAF'    => $imp['VRIMPITMAF'],
            'VRPERREDBCAF'  => $imp['VRPERREDBCAF'],
            'VRBASECALCAF'  => $imp['VRBASECALCAF'],
            'VRALIQITMAF'   => $imp['VRALIQITMAF'],
            'VRPARCDEDAF'   => $imp['VRPARCDEDAF'],
            'CDFISCITMAF'   => $imp['CDFISCITMAF'],
            'VRADIMPAF'     => $imp['VRADIMPAF'],
            'IDIMPDIGAF'    => $imp['IDIMPDIGAF']
        );
        
        $query = "INSERT INTO IMPITMAF
                    (CDFILDEST,  CDFILLANC, NRAUTOFORN,
                    CDPRODUTO, CDMARCA, DTPROGENTAF,
                    CDIMPOSTO, VRIMPITMAF, VRPERREDBCAF,
                    VRBASECALCAF, VRALIQITMAF, VRPARCDEDAF,
                    CDFISCITMAF, VRADIMPAF, IDIMPDIGAF)
                    SELECT :CDFILDEST,  :CDFILLANC, :NRAUTOFORN,
                    :CDPRODUTO, :CDMARCA, :DTPROGENTAF,
                    :CDIMPOSTO, :VRIMPITMAF, :VRPERREDBCAF,
                    :VRBASECALCAF, :VRALIQITMAF, :VRPARCDEDAF,
                    :CDFISCITMAF, :VRADIMPAF, :IDIMPDIGAF
                    FROM PARAMGERAL
                    WHERE NOT EXISTS(SELECT 1
                                       FROM IMPITMAF I
                                       WHERE I.CDFILDEST   = :CDFILDEST
                                         AND I.CDFILLANC   = :CDFILLANC
                                         AND I.NRAUTOFORN  = :NRAUTOFORN
                                         AND I.CDPRODUTO   = :CDPRODUTO
                                         AND I.CDMARCA     = :CDMARCA
                                         AND I.DTPROGENTAF = :DTPROGENTAF
                                         AND I.CDIMPOSTO   = :CDIMPOSTO)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function sqlWebTunning() {
        $param = array();
        
        $query = "SELECT
                       MAX(NVL(DSREMOTEADDR, 'VIA BD')) DSREMOTEADDR,
                       MAX(NVL(DSREMOTEHOST, 'VIA BD')) DSREMOTEHOST,
                       MAX(NVL(DSSERVERRDM, 'VIA BD'))  DSSERVERRDM,
                       MAX(NVL(DSSERVMETHOD, 'VIA BD')) DSSERVMETHOD,
                       MAX(NVL(DSREMOTEOPER, 'VIA BD')) DSREMOTEOPER
                    FROM
                       SQLWEBTUNNING
                    WHERE
                       NRAUDSID = USERENV('SESSIONID') AND
                       TRUNC(DTSTARTDATE) = (SELECT MAX(TRUNC(DTSTARTDATE))
                                             FROM SQLWEBTUNNING S
                                             WHERE S.NRAUDSID = USERENV('SESSIONID'))";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsMovOpAF($row, $sqlWT, $msgJust, $stIdOperacao) {
        $param = array(
            'CDFILDESMOV' => $row['CDFILDEST'],
            'CDFILLANMOV' => $row['CDFILLANC'],
            'NRAUTOMOV' => $row['NRAUTOFORN'],
            'DTPROGENTMOV' => substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'CDPRODMOV' => $row['CDPRODUTO'],
            'CDMARCMOV' => $row['CDMARCA'],
            'DSJUSTOPERMOV' => $msgJust,
            'CDOPERADOR' => $this->environment->getCdOperador(),
            'DSALTERAF' => $msgJust,
            'IDOPERAMOVAF' => $stIdOperacao,
            'DSRDMUSUMOVAF' => $sqlWT['DSSERVERRDM'],
            'DSENDUSUMOVAF' => $sqlWT['DSREMOTEADDR'],
            'DSNMUSUMOVAF' => $sqlWT['DSREMOTEHOST'],
            'DSMETUSUMOVAF' => $sqlWT['DSSERVMETHOD']
        );
        
        $query = "insert into MOVOPAF
                      (CDFILDESMOV, CDFILLANMOV, NRAUTOMOV,  CDPRODMOV,    CDMARCMOV, DTPROGENTMOV, DTOPERAF,  CDOPERADOR,     DSALTERAF, DSJUSTOPERMOV,
                       IDOPERAMOVAF ,  DSRDMUSUMOVAF, DSENDUSUMOVAF,    DSNMUSUMOVAF, DSMETUSUMOVAF)
                    values
                      (:CDFILDESMOV, :CDFILLANMOV, :NRAUTOMOV, :CDPRODMOV,    :CDMARCMOV, :DTPROGENTMOV, SYSDATE, :CDOPERADOR,    :DSALTERAF,
                       :DSJUSTOPERMOV,      :IDOPERAMOVAF ,    :DSRDMUSUMOVAF, :DSENDUSUMOVAF,  :DSNMUSUMOVAF,      :DSMETUSUMOVAF)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsMovOpAF1($row, $sqlWT, $msgJust, $stIdOperacao) {
        $param = array(
            'CDFILDESMOV' => $row['CDFILDEST'],
            'CDFILLANMOV' => $row['CDFILLANC'],
            'NRAUTOMOV' => $row['NRAUTOFORN'],
            'DTPROGENTMOV' => $stIdOperacao == 'E' ? substr($row['DTPROGENTAF'], 0, 10) : substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'CDPRODMOV' => $row['CDPRODUTO'],
            'CDMARCMOV' => $row['CDMARCA'],
            'DSJUSTOPERMOV' => $msgJust,
            'CDOPERADOR' => $this->environment->getCdOperador(),
            'DSALTERAF' => $msgJust,
            'IDOPERAMOVAF' => $stIdOperacao,
            'DSRDMUSUMOVAF' => $sqlWT['DSSERVERRDM'],
            'DSENDUSUMOVAF' => $sqlWT['DSREMOTEADDR'],
            'DSNMUSUMOVAF' => $sqlWT['DSREMOTEHOST'],
            'DSMETUSUMOVAF' => $sqlWT['DSSERVMETHOD']
        );
        
        $query = "insert into MOVOPAF
                      (CDFILDESMOV, CDFILLANMOV, NRAUTOMOV,  CDPRODMOV,    CDMARCMOV, DTPROGENTMOV, DTOPERAF,  CDOPERADOR,     DSALTERAF, DSJUSTOPERMOV,
                       IDOPERAMOVAF ,  DSRDMUSUMOVAF, DSENDUSUMOVAF,    DSNMUSUMOVAF, DSMETUSUMOVAF)
                    values
                      (:CDFILDESMOV, :CDFILLANMOV, :NRAUTOMOV, :CDPRODMOV,    :CDMARCMOV, :DTPROGENTMOV, SYSDATE, :CDOPERADOR,    :DSALTERAF,
                       :DSJUSTOPERMOV,      :IDOPERAMOVAF ,    :DSRDMUSUMOVAF, :DSENDUSUMOVAF,  :DSNMUSUMOVAF,      :DSMETUSUMOVAF)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsLogPlanItemAF($row, $msgJust, $stIdOperacao) {
        $param = array(
            'P_NRLOGPLANITEMAF' => self::procedureGeraNovoCodigo('LOGPLANITEMAF'),
            'P_CDFILDEST' => $row['CDFILDEST'],
            'P_CDFILLANC' => $row['CDFILLANC'],
            'P_NRAUTOFORN' => $row['NRAUTOFORN'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDMARCA' => $row['CDMARCA'],
            'P_DTPROGENTAFANT' => substr($row['DTPROGENTAF'], 0, 10),
            'P_DTPROGENTAFDEP' => substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'P_DSJUSTOPERMOV' => $msgJust,
            'P_QTPROGENTANT' => self::parseFloat($row['QTORIG']),
            'P_QTPROGENTDEP' => $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG'])
        );
        
        $query = "INSERT
                      INTO LOGPLANITEMAF
                           (NRLOGPLANITEMAF, CDFILDEST, CDFILLANC, NRAUTOFORN, CDMARCA, CDPRODUTO,
                           DTPROGENTAFANT, DTPROGENTAFDEP, QTPROGENTANT, QTPROGENTDEP, DSJUSTITMAF, DTOPERAF)
                    VALUES (:P_NRLOGPLANITEMAF, :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN,:P_CDMARCA, :P_CDPRODUTO,
                            :P_DTPROGENTAFANT, :P_DTPROGENTAFDEP, :P_QTPROGENTANT, :P_QTPROGENTDEP, :P_DSJUSTOPERMOV, SYSDATE)";
                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsLogPlanItemAF1($row, $msgJust, $stIdOperacao) {
        $param = array(
            'P_NRLOGPLANITEMAF' => self::procedureGeraNovoCodigo('LOGPLANITEMAF'),
            'P_CDFILDEST' => $row['CDFILDEST'],
            'P_CDFILLANC' => $row['CDFILLANC'],
            'P_NRAUTOFORN' => $row['NRAUTOFORN'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDMARCA' => $row['CDMARCA'],
            'P_DTPROGENTAFANT' => substr($row['DTPROGENTAF'], 0, 10),
            'P_DTPROGENTAFDEP' => $stIdOperacao == 'E' ? null : $stIdOperacao == 'E' ? substr($row['DTPROGENTAF'], 0, 10) : substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'P_DSJUSTOPERMOV' => $msgJust,
            'P_QTPROGENTANT' => self::parseFloat($row['QTORIG']),
            'P_QTPROGENTDEP' => $stIdOperacao == 'E' ? 0 : $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG'])
        );
        
        $query = "INSERT
                      INTO LOGPLANITEMAF
                           (NRLOGPLANITEMAF, CDFILDEST, CDFILLANC, NRAUTOFORN, CDMARCA, CDPRODUTO,
                           DTPROGENTAFANT, DTPROGENTAFDEP, QTPROGENTANT, QTPROGENTDEP, DSJUSTITMAF, DTOPERAF)
                    VALUES (:P_NRLOGPLANITEMAF, :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN,:P_CDMARCA, :P_CDPRODUTO,
                            :P_DTPROGENTAFANT, :P_DTPROGENTAFDEP, :P_QTPROGENTANT, :P_QTPROGENTDEP, :P_DSJUSTOPERMOV, SYSDATE)";
                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
}